
import BingoGoal from '@/js/lib/BingoGoal.ts';
import BingoGroup from '@/js/lib/BingoGroup.ts';
import type SuccessResponse from '@/js/lib/SuccessResponse.ts';
import Util from '@/js/lib/Util.ts';
import {
	Type,
	instanceToPlain
} from 'class-transformer';

export default class BingoGame {
	id: string;

	name: string;

	short_description: string;

	description: string;

	generator: 'simple' | 'srl_v5' | 'srl_v8' = 'simple';

	@Type(() => BingoGoal)
		goals: BingoGoal[] = [];

	@Type(() => BingoGroup)
		groups: BingoGroup[] = [];

	constructor(
		id: string,
		name: string,
		short_description: string,
		description: string
	) {
		this.id = id;
		this.name = name;
		this.short_description = short_description;
		this.description = description;
	}

	addGoal(goal: BingoGoal): SuccessResponse {
		const existingGoal = this.goals.find(g => g === goal);
		if (existingGoal) {
			return Util.returnSuccess(false, 'A goal with this name already exists');
		}
		else {
			this.goals.push(goal);

			return Util.returnSuccess(true);
		}
	}

	removeGoal(goal: BingoGoal): SuccessResponse {
		const filteredGoals = this.goals.filter(g => g !== goal);
		if (filteredGoals.length === this.goals.length) {
			return Util.returnSuccess(false, 'Goal not found');
		}
		else {
			this.goals = filteredGoals;

			return Util.returnSuccess(true);
		}
	}

	removeGoalByName(name: string): SuccessResponse {
		const goal = this.goals.find(g => g.name === name);
		if (!goal) {
			return Util.returnSuccess(false, 'Goal not found');
		}

		return this.removeGoal(goal);
	}

	addGroup(group: BingoGroup): SuccessResponse {
		const existingGroup = this.groups.find(g => g === group);
		if (existingGroup) {
			return Util.returnSuccess(false, 'A group with this name already exists');
		}
		else {
			this.groups.push(group);

			return Util.returnSuccess(true);
		}
	}

	removeGroup(group: BingoGroup): SuccessResponse {
		const filteredGroups = this.groups.filter(g => g !== group);
		if (filteredGroups.length === this.groups.length) {
			return Util.returnSuccess(false, 'Group not found');
		}
		else {
			// Check if any group has the given group as its parent
			const hasParent = this.groups.some(g => g.parent === group);
			if (hasParent) {
				// Add the group back to the list of groups
				this.groups.push(group);

				return Util.returnSuccess(false, 'Cannot delete group as it is a parent of another group');
			}
			else {
				this.groups = filteredGroups;

				return Util.returnSuccess(true);
			}
		}
	}

	removeGroupByName(name: string): SuccessResponse {
		const group = this.groups.find(g => g.name === name);
		if (!group) {
			return Util.returnSuccess(false, 'Group not found');
		}

		return this.removeGroup(group);
	}

	getGoalsByTags(...tags: string[]): BingoGoal[] {
		return this.goals.filter(goal =>
			tags.some(
				tag => goal.tags.some(
					goalTag => Util.stringCompare(goalTag, tag)
				)
			));
	}

	getGoalsByGroup(includeParents: boolean, ...groups: BingoGroup[]): BingoGoal[] {
		const result: BingoGoal[] = [];

		this.goals.map(goal => {
			const goal_group = goal.group;

			if (includeParents) {
				let parent = goal_group.parent;
				while (parent) {
					if (groups.includes(parent)) {
						result.push(goal);

						return;
					}

					parent = parent.parent;
				}
			}

			if (groups.includes(goal_group)) {
				result.push(goal);
			}
		});

		return result;
	}

	toJSON(): Record<string, any> {
		this.groups.sort((a, b) => {
			if (!a.parent && b.parent) {
				return -1;
			}

			if (a.parent && !b.parent) {
				return 1;
			}

			return 0;
		});

		return instanceToPlain(this);
	}
}
